//Software for synth with Arduino UNO on shieldPCB Mvoxs1b
// structure: Modulators>(pulseosc+subosc+wave/pattern/noise) ->VCA1 -> VCF ->out
//reads MIDI-input on pin RX0 via optocoupler, NO pull-up 1k6
//Controls analog VCF lowpass filter (PoliVoks style) Frequency (exp) and Q (up to osc): 
//Controls analog VCA:  VCA before the VCF
//Generates envelope and LFO waveform voltages via 2 modulators
//generates pulse wave with hardware PWM, 6bit Binairy-Trinairy DAC (216 values) for subosc, waves and noise
// DAC= R-Dac with 3 lower bits trinair (using tristate via DDR as input) + 3 higher bits binair
// DAC R-values (lsb-msb):1M-330k-110k-75k-36k-18k connected to the node of 1k-to-ground and 1k-to-5V
// LED connections to panel:
// D08=LV1,D09=LV2,D12=LM1,D04=LM2,D02=LOS,D07=LSU,D11=LA1,D06=LFF,D05=LFQ,D03=LA2
// MIDI control of all parameters
// ideas:Subosc wave:noise>>done, variable length prbs(M2tone>length), M1level>M2rate, sampling on dac
// ideas: Modulator wave, sampling on dac or timer1+sub state., lpf added
// ideas:Play mode:normal/glide(rate)/decay(rate)/hold>>done
// ideas: M2 depth mid (or mostleft) position is Arpeggio, wave sets arp mode, rate sets speed, pattern sets variation
// ideas: when mod is in env mode, key off freezes value or key off goes to 2nd slope(decay)
// ideas:percmode/waveseq recall patch, percmode:prog65=each key is patch
// ideas: accent pattern always 1 at keystart, pw up to 0% when via m1 
// ideas: wave play with 256point and skip points at higher octaves >>done
// ideas: LDR right "hit"is trigger note, left is lower volume or lower filter
// ideas: reverb effect like tx81z, long release at low level >>done
// ideas: Pulse as waveform, use 8 segments, amplitude of each segment is a variable that can be modulated
// PWM for Q at higher freq, rest at subdiv >>done
// todo: midi clockrate interpolator F>4F >>done
// todo: move glide function to notesubroutine (works also with modulators then) 
// todo: 128 length table, now 32,  for LV1 and LV2 to mark end points>>done
// todo: Modulator wavemode that is key-on gated, freeze at key-off
#include <EEPROM.h>
#include <avr/pgmspace.h>
// MIDI handling variables:
boolean MIDIcon=false;// on boot local control, on detecting MIDIevents switch to MIDIcon=true, no local control
byte incomingByte=0;
byte statusByte=0;
byte dataByte=0;
byte statusbuffer=0;
byte CHAN_NR=1; //receive midi channel is 1, is set here
byte NOTE_ON=144+CHAN_NR-1;
byte NOTE_OFF=128+CHAN_NR-1;
byte CONT_CHANGE=176+CHAN_NR-1;
byte PROG_CHANGE=192+CHAN_NR-1;//program change 
byte PITCHW_CHANGE=224+CHAN_NR-1;
byte AFT_CHANGE=208+CHAN_NR-1;
byte LAST_NOTE=41;
//byte OLD_NOTE=0; // for arpeggio mode
byte GLIDE_NOTE=36;//used for portamento mode to glide from old to new note
byte GlideTm=0;//sets portamento time
byte LAST_PROG=0;//patch program nr
byte LAST_VEL=70;//key velocity
byte LAST_MWH=48;//modulation wheel
byte LAST_PWH=64;//pitch bend wheel
byte LAST_AFT=0;//after touch (channelmode)
boolean MidiClk=false;//true if a MidiClock signal present, switch to ext clock mode
boolean MidiTick=false;//true if MidiClockTick received
unsigned int Mclkper=200;//MIDIclock period measured
unsigned int Mclktim=0;// MIDIclock period timer
boolean firstbyte;
boolean KEY_DOWN=1; // key down state
boolean KEY_START=1; // key start state
byte KEY_STARTcnt=0;//counts ticks after at keystart
//midi controllers:
byte PotmOm=46;//Pot for Osc mode
byte PotmAv2=127; //vca 2 midi channel volume
byte LAST_PotmAv2=0;
byte PotmAm=0;//VCA envelope mode
//byte PotmAv1=63; //vca1 volume
byte PotmOw=45; // Osc wave Pulse/PWM/Saw/Sine/Pattern/Noise
byte LAST_PotmOw=0;
//byte PotmOwP=45;
byte PotmKm=0; //keyboard mode, 0=no portamento, 0..14=portamento time >14 vibrato
byte LAST_PotmKm=0;//
byte PotmM1r=46; // Modulator1 rate
byte LAST_PotmM1r=0;
byte PotmM1d=86; // Modulator1 depth
byte PotmM1s=0;// Modulator1 shape select
byte PotmM1p=0; // Modulator1 accent pattern
byte PotmFf=56;  // Filter freq
byte PotmFm=20;  // Filter mode
byte LAST_PotmFm=1;
byte PotmM2r=105; // Modulator2 rate
byte LAST_PotmM2r=0;
byte PotmM2d=84;  // Modulator2 depth
byte PotmM2s=0;// Modulator2 shape
byte PotmM2p=21; // Modulator2 depth pattern
byte PotmSw=127;  // SubOsc wave :noise, pattern,F/16...F/2
byte LAST_PotmSw=0;
byte PotmSm=0;  // SubOsc mode select OFF/level/max/ min-FilterFM-max/AM
byte LAST_PotmSm=0; 
byte PotmKvib=0;//vibrato pot
byte PotmKprt=0;//portamento pot
boolean PB1=1;//pushbutton SW1 key
boolean LAST_PB1=1;//pushbutton SW1, last value
boolean PB2=1;//pushbutton SW2 program+
boolean LAST_PB2=1;//pushbutton SW2, last value
boolean PB3=1;//pushbutton SW3 program-
boolean LAST_PB3=1;//pushbutton SW3, last value
//variables
byte PW; // Osc Pulse pulsewidth value
byte Note; //MIDI note number
byte nt; // temp note variable in subroutine NOTE to Freq
byte oct=1;//octave number for frequency calculation, lowest oct=1, means C=16.35Hz
//byte octMod=0; //modulator for creating harmonics 0 =no harmonics
// Oscillator timer divider settings for musical scale, 12 notes, oct number sets div/2^oct, prescaler 8>1 for pulse>dac :
unsigned int Note_div_array[12]={61162,57737,54496,51414,48544,45809,43243,40825,38525,36364,34323,32396};
//C=16.35Hz,C#17.32Hz,D18.35Hz,D#19.45Hz,E20.60Hz,F21.83Hz,F#23.125Hz,G24.495Hz,G#25.957Hz,A27.50Hz,A#29.135Hz,B30.868Hz
//Filter:
int filterF=0;// Filterfreq
//byte QAtab[32]={16,14,14,14,14,14,14,14, 14,14,14,14,14,14,14,14, 14,14,14,10,5,0,0,0, 0,0,0,0,2,4,8,45 }; //amplitude correction for Q
//byte QAtab[32]PROGMEM ={16,14,12,12,12,12,12,12, 12,12,12,12,14,14,14,14, 14,14,14,12,8,4,0,0, 0,0,0,0,5,15,40,60 }; //amplitude correction for Q
//Amplifier:
//amp curve:
byte VCAtab[128]PROGMEM ={0,5,10,15,20,25,26,26, 27,27,28,28,29,29,30,30, 30,31,31,31,32,32,32,33,33, 33,34,34,34,35,35,35, 
36,36,36,37,37,37,38,38, 38,39,39,39,40,40,40,41, 41,41,42,42,42,43,43,44, 44,45,45,46,46,47,47,48, 
48,49,49,50,50,51,51,52, 52,53,53,54,54,55,55,56, 56,57,57,58,59,60,61,62, 63,64,65,66,67,68,69,70,
71,72,73,74,75,76,78,80, 82,84,86,88,90,92,94,97, 100,110,120,130,140,150,160,170, 180,190,200,210,220,230,240,255 };
byte VCA2=0;
// amplitude attenuation versus key
byte VCAkey[120]PROGMEM ={0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,
1,2,2,3,3,4,4,5, 5,6,6,7,7,8,8,9, 9,10,10,1,11,12,12,13, 14,15,16,17,18,19,20,21, 22,23,24,25,26,27,28,29, 30,30,30,30,30,30,30,30,
 30,30,30,30,30,30,30,30, 30,30,30,30,30,30,30,30, 30,30,30,30,30,30,30,30};
byte ampDcy=0;
byte ampEnv=128;//amp envelope value 128=max 0=off
//Noise generator:
word state =1; // Pseudo Random Bit Sequence seed value
//word LAST_state =1; // Pseudo Random Bit Sequence seed value
word NoiseLen=1000;
word NoiseCnt=0;
//byte NoiseF=0;
//word NoiseLim=1;
word NoiseLentab[35]= {8,10,12,14, 16,18,20,22,24,26,28,30,32,36, 38,40,
                       50,64,96,128,256,384,512,1000,2000,3000,4000,5000,6000,8000,12000,16000,20000,24000,32000};
byte DACP=0;//DAC table pointer value
byte WavP =0; //pointer in waveform
byte LAST_WavP=0;
byte WavP2 =0; //2nd pointer in waveform for waveshapemodulation
//byte WavPmod=0;//modulatorvalue for pointer 
//boolean WavStart=true;
byte WavInctab [8] = {1, 1, 1, 2, 4, 8,16,32};//increment size in waveform stepping
byte WavSkiptab [8] ={3, 1, 0, 0, 0, 0, 0, 0};//skip numbers for increment in waveform stepping
byte WavSkip=0; //counter for skipping of increments

//DAC waveforms:
byte DAC=8;
byte WavMode=0; // 0=OSC off, 1=OSC in pulse mode, 2=wave mode 3=pattern mode, 4=noise mode
unsigned int WavSel=0;// pointer for waveselect
unsigned int WavSelD=0;// pointer for waveselect, dynamic sync updated with waveform start
//256x7bit waveforms used for waveform output on pins A0-A5:
byte WavTab[4096] PROGMEM={
//SawF1L96  means: saw frequency=1 level=96
0,0,0,1,1,1,2,2,3,3,3,4,4,4,5,5,6,6,6,7,7,7,8,8,9,9,9,10,10,10,11,11,12,12,12,13,13,13,14,
14,15,15,15,16,16,16,17,17,18,18,18,19,19,19,20,20,21,21,21,22,22,22,23,23,24,24,24,25,
25,25,26,26,27,27,27,28,28,28,29,29,30,30,30,31,31,31,32,32,33,33,33,34,34,34,35,35,36,
36,36,37,37,37,38,38,39,39,39,40,40,40,41,41,42,42,42,43,43,43,44,44,45,45,45,46,46,46,47,
47,48,48,48,49,49,49,50,50,51,51,51,52,52,52,53,53,54,54,54,55,55,55,56,56,57,57,57,58,
58,58,59,59,60,60,60,61,61,61,62,62,63,63,63,64,64,64,65,65,66,66,66,67,67,67,68,68,69,
69,69,70,70,70,71,71,72,72,72,73,73,73,74,74,75,75,75,76,76,76,77,77,78,78,78,79,79,79,80,
80,81,81,81,82,82,82,83,83,84,84,84,85,85,85,86,86,87,87,87,88,88,88,89,89,90,90,90,91,
91,91,92,92,93,93,93,94,94,94,95,95,

//SawF1L64SinF2L24
24,23,22,20,20,19,18,17,17,16,15,14,14,13,12,11,11,10,9,9,9,8,8,7,8,7,7,7,7,7,7,7,8,8,8,
8,9,10,10,10,12,12,13,13,15,16,16,17,19,20,21,22,24,25,26,27,29,30,31,32,34,35,37,38,40,
41,42,44,46,47,48,49,51,52,53,54,56,57,58,59,61,62,63,63,65,66,66,67,68,69,69,69,71,71,71,
71,72,72,72,72,73,72,72,72,72,72,71,71,71,70,70,69,69,68,67,66,66,65,64,63,63,62,61,60,
60,59,57,56,56,55,54,52,52,51,50,49,49,48,47,46,46,45,44,43,43,42,41,41,41,40,40,39,40,
39,39,39,39,39,39,39,40,40,40,40,41,42,42,42,44,44,45,45,47,48,48,49,51,52,53,54,56,57,58,
59,61,62,63,64,66,67,69,70,72,73,74,76,78,79,80,81,83,84,85,86,88,89,90,91,93,94,95,95,
97,98,98,99,100,101,101,101,103,103,103,103,104,104,104,104,105,104,104,104,104,104,103,
103,103,102,102,101,101,100,99,98,98,97,96,95,95,94,93,92,92,91,89,88,

//SawF1L64SinF3L24
24,22,20,19,18,16,15,13,13,11,10,9,8,7,6,6,6,5,5,4,5,5,5,5,6,7,7,8,10,11,12,13,15,16,18,
19,22,24,25,27,30,31,33,35,38,39,41,43,45,47,49,50,53,54,55,56,58,59,60,61,62,63,63,63,64,
64,64,63,64,63,62,62,62,61,59,58,58,56,55,53,53,51,49,48,47,45,43,41,41,39,37,36,35,34,
32,31,31,29,28,27,28,27,26,26,26,26,26,26,27,27,28,28,30,31,32,33,35,36,37,39,41,43,45,
46,49,51,52,54,57,59,60,62,65,66,68,70,72,74,75,76,78,79,80,81,83,83,84,84,85,85,85,85,85,
85,84,83,84,83,82,80,80,79,77,76,75,74,72,70,70,68,66,64,63,62,60,58,58,56,55,53,53,52,
50,49,49,49,48,47,48,47,47,47,48,48,48,49,50,51,52,53,55,56,57,58,61,62,64,66,68,70,72,
73,76,78,80,81,84,86,87,89,92,93,95,96,98,99,100,101,103,104,104,105,106,106,106,106,107,
106,106,105,105,105,104,103,102,101,100,98,98,96,95,93,92,91,89,87,

//SawF1L64SinF4L24
24,22,19,17,16,14,12,10,9,7,6,5,5,4,3,3,4,4,4,5,7,8,9,10,13,15,17,19,22,24,26,29,32,34,37,
39,42,44,46,48,51,53,54,55,57,58,59,59,60,60,60,59,59,58,57,56,55,53,51,49,48,46,44,41,
40,38,35,33,32,30,28,26,25,23,22,21,21,20,19,19,20,20,20,21,23,24,25,26,29,31,33,35,38,
40,42,45,48,50,53,55,58,60,62,64,67,69,70,71,73,74,75,75,76,76,76,75,75,74,73,72,71,69,67,
65,64,62,60,57,56,54,51,49,48,46,44,42,41,39,38,37,37,36,35,35,36,36,36,37,39,40,41,42,
45,47,49,51,54,56,58,61,64,66,69,71,74,76,78,80,83,85,86,87,89,90,91,91,92,92,92,91,91,
90,89,88,87,85,83,81,80,78,76,73,72,70,67,65,64,62,60,58,57,55,54,53,53,52,51,51,52,52,52,
53,55,56,57,58,61,63,65,67,70,72,74,77,80,82,85,87,90,92,94,96,99,101,102,103,105,106,
107,107,108,108,108,107,107,106,105,104,103,101,99,97,96,94,92,89,

//SawF1L64SinF5L24
24,21,18,15,14,11,9,7,6,5,3,3,3,3,3,4,6,7,9,11,14,16,19,22,26,29,31,34,38,41,44,46,49,51
,53,54,56,57,57,57,57,57,55,54,53,51,49,46,45,42,39,36,34,31,28,26,24,22,20,18,18,16,16,
15,16,16,17,18,20,22,24,26,29,32,35,38,42,44,47,50,54,57,59,61,64,66,67,68,70,70,70,69,70,
68,67,65,64,62,59,57,55,52,49,46,44,41,39,36,35,33,31,30,29,29,28,28,29,30,31,32,35,37,39,
41,45,48,51,54,57,60,63,66,70,72,74,76,79,80,81,82,83,83,82,82,81,80,78,76,75,72,70,
67,65,62,59,56,54,52,49,47,46,44,43,41,42,41,41,41,43,44,45,47,50,52,54,57,61,64,67,69,73,
76,79,82,85,87,89,91,93,94,95,95,96,95,95,93,93,91,89,87,85,83,80,77,75,72,69,66,65,62,
60,58,57,56,54,54,54,54,54,55,57,58,60,62,65,67,70,73,77,80,82,85,89,92,95,97,100,102,104,
105,107,108,108,108,108,108,106,105,104,102,100,97,96,93,90,87,

//SawF1L64SinF6L24
24,20,17,14,11,9,6,4,4,3,2,2,4,5,6,8,12,14,18,21,25,29,33,36,40,44,46,49,52,53,54,55,56,
55,54,53,52,49,47,43,41,38,34,30,28,25,21,19,17,15,14,13,13,13,14,15,17,19,22,24,29,32,35,
39,44,47,50,54,57,60,62,63,65,66,66,65,65,64,62,59,58,54,51,48,45,41,38,35,32,30,27,25,25,
24,23,23,25,26,27,29,33,35,39,42,46,50,54,57,61,65,67,70,73,74,75,76,77,76,75,74,73,
70,68,64,62,59,55,51,49,46,42,40,38,36,35,34,34,34,35,36,38,40,43,45,50,53,56,60,65,68,71,
75,78,81,83,84,86,87,87,86,86,85,83,80,79,75,72,69,66,62,59,56,53,51,48,46,46,45,44,44,46,
47,48,50,54,56,60,63,67,71,75,78,82,86,88,91,94,95,96,97,98,97,96,95,94,91,89,85,83,
80,76,72,70,67,63,61,59,57,56,55,55,55,56,57,59,61,64,66,71,74,77,81,86,89,92,96,99,102,
104,105,107,108,108,107,107,106,104,101,100,96,93,90,87,83,80,77,

//SawF1L64SinF7L24
24,20,16,12,10,7,4,2,2,2,2,3,6,9,12,15,20,24,28,32,37,41,44,47,51,53,54,54,55,54,52,49,47,
44,40,36,33,29,25,21,19,16,13,11,11,11,11,12,15,18,21,24,29,33,37,41,46,50,53,56,60,62,
63,63,64,63,61,58,56,53,49,45,42,38,34,30,28,25,22,20,20,20,20,21,24,27,30,33,38,42,46,
50,55,59,62,65,69,71,72,72,73,72,70,67,65,62,58,54,51,47,43,39,37,34,31,29,29,29,29,30,33,
36,39,42,47,51,55,59,64,68,71,74,78,80,81,81,82,81,79,76,74,71,67,63,60,56,52,48,46,43,
40,38,38,38,38,39,42,45,48,51,56,60,64,68,73,77,80,83,87,89,90,90,91,90,88,85,83,80,76,
72,69,65,61,57,55,52,49,47,47,47,47,48,51,54,57,60,65,69,73,77,82,86,89,92,96,98,99,99,
100,99,97,94,92,89,85,81,78,74,70,66,64,61,58,56,56,56,56,57,60,63,66,69,74,78,82,86,91,
95,98,101,105,107,108,108,109,108,106,103,101,98,94,90,87,83,79,75,

//SawF1L64SinF8L24
24,19,15,11,8,5,3,1,2,2,4,6,10,14,18,22,28,33,37,41,46,49,51,53,54,54,52,50,48,44,40,36,
32,27,23,19,16,13,11,9,10,10,12,14,18,22,26,30,36,41,45,49,54,57,59,61,62,62,60,58,56,52,
48,44,40,35,31,27,24,21,19,17,18,18,20,22,26,30,34,38,44,49,53,57,62,65,67,69,70,70,68,
66,64,60,56,52,48,43,39,35,32,29,27,25,26,26,28,30,34,38,42,46,52,57,61,65,70,73,75,77,78,
78,76,74,72,68,64,60,56,51,47,43,40,37,35,33,34,34,36,38,42,46,50,54,60,65,69,73,78,81,83,
85,86,86,84,82,80,76,72,68,64,59,55,51,48,45,43,41,42,42,44,46,50,54,58,62,68,73,77,
81,86,89,91,93,94,94,92,90,88,84,80,76,72,67,63,59,56,53,51,49,50,50,52,54,58,62,66,70,76,
81,85,89,94,97,99,101,102,102,100,98,96,92,88,84,80,75,71,67,64,61,59,57,58,58,60,62,66,
70,74,78,84,89,93,97,102,105,107,109,110,110,108,106,104,100,96,92,

//SinF1L48
48,49,50,52,53,54,55,56,57,59,60,61,62,63,64,65,66,67,69,70,71,72,73,74,75,76,77,78,78,79,
80,81,82,83,84,84,85,86,87,87,88,89,89,90,90,91,91,92,92,93,93,94,94,94,95,95,95,95,95,96,
96,96,96,96,96,96,96,96,96,96,95,95,95,95,95,94,94,94,93,93,92,92,91,91,90,90,89,89,88,87,
87,86,85,84,84,83,82,81,80,79,78,78,77,76,75,74,73,72,71,70,69,67,66,65,64,63,62,61,60,59,
57,56,55,54,53,52,50,49,48,47,46,44,43,42,41,40,39,37,36,35,34,33,32,31,30,29,27,26,25,24,
23,22,21,20,19,18,18,17,16,15,14,13,12,12,11,10,9,9,8,7,7,6,6,5,5,4,4,3,3,2,2,2,1,1,1,1,1,0,0,
0,0,0,0,0,0,0,0,0,1,1,1,1,1,2,2,2,3,3,4,4,5,5,6,6,7,7,8,9,9,10,11,12,12,13,14,15,16,17,18,18,19,
20,21,22,23,24,25,26,27,29,30,31,32,33,34,35,36,37,39,40,41,42,43,44,46,47,

//SinF1L32SinF2L16
48,50,51,53,54,56,57,59,60,62,63,65,66,68,69,70,72,73,74,75,76,77,79,80,81,81,82,83,84,85,
85,86,87,87,88,88,88,89,89,89,89,89,90,90,90,89,89,89,89,89,88,88,88,87,87,86,86,85,84,
84,83,82,82,81,80,79,78,78,77,76,75,74,73,72,71,71,70,69,68,67,66,65,65,64,63,62,61,61,
60,59,58,58,57,56,56,55,55,54,54,53,53,52,52,51,51,51,50,50,50,50,49,49,49,49,49,49,48,48,
48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,48,47,47,47,47,47,47,
46,46,46,46,45,45,45,44,44,43,43,42,42,41,41,40,40,39,38,38,37,36,35,35,34,33,32,31,31,
30,29,28,27,26,25,25,24,23,22,21,20,19,18,18,17,16,15,14,14,13,12,12,11,10,10,9,9,8,8,8,
7,7,7,7,7,6,6,6,7,7,7,7,7,8,8,8,9,9,10,11,11,12,13,14,15,15,16,17,19,20,21,22,23,24,26,27,
28,30,31,33,34,36,37,39,40,42,43,45,46,

//SinF1L32SinF3L16
48,50,52,54,56,58,60,61,63,65,67,68,70,71,73,74,75,76,77,78,79,80,80,81,81,82,82,82,82,82,
82,82,82,82,81,81,80,80,79,78,78,77,76,75,74,74,73,72,71,70,70,69,68,68,67,66,66,65,65,65,
64,64,64,64,64,64,64,64,65,65,65,66,66,67,67,68,69,69,70,71,72,73,73,74,75,76,77,77,
78,79,79,80,81,81,82,82,82,82,83,83,83,82,82,82,82,81,80,80,79,78,77,76,75,74,72,71,69,68,
66,64,63,61,59,57,55,53,51,49,47,45,43,42,40,38,36,34,32,31,29,27,26,25,23,22,21,20,19,18,
17,16,16,15,15,14,14,14,14,14,14,14,15,15,15,16,16,17,18,18,19,20,21,21,22,23,24,25,
25,26,27,27,28,29,29,30,30,31,31,31,32,32,32,32,32,32,32,31,31,31,30,30,29,29,28,27,27,26,
25,24,24,23,22,21,20,20,19,18,17,17,16,15,15,14,14,14,13,13,13,13,13,13,14,14,14,15,16,16,
17,18,19,20,21,23,24,26,27,29,30,32,34,36,38,39,41,43,45,47,

//SinF1L32SinF4L16
48,50,53,55,57,59,62,64,66,67,69,71,72,73,74,75,76,77,77,78,78,78,78,77,77,77,76,75,74,74,
73,72,71,70,69,68,67,66,65,64,63,63,62,62,61,61,61,61,62,62,62,63,64,65,66,67,68,69,71,72,
74,75,77,78,80,82,83,85,86,87,89,90,91,92,92,93,93,94,94,94,94,93,93,92,91,90,89,87,
86,84,83,81,79,77,75,73,71,68,66,64,62,60,58,56,54,53,51,50,48,47,46,45,44,44,43,43,43,42,
42,43,43,43,44,44,45,46,46,47,48,49,50,50,51,52,52,53,53,53,54,54,53,53,53,52,52,51,50,
49,48,46,45,43,42,40,38,36,34,32,30,28,25,23,21,19,17,15,13,12,10,9,7,6,5,4,3,3,2,2,2,2,3,
3,4,4,5,6,7,9,10,11,13,14,16,18,19,21,22,24,25,27,28,29,30,31,32,33,34,34,34,35,35,35,35,
34,34,33,33,32,31,30,29,28,27,26,25,24,23,22,22,21,20,19,19,19,18,18,18,18,19,19,20,
21,22,23,24,25,27,29,30,32,34,37,39,41,43,46,

//SinF1L32SinF5L16
48,51,53,56,59,61,63,66,68,69,71,72,73,74,75,75,75,75,74,74,73,72,71,70,69,67,66,65,63,62,
61,60,59,58,58,57,57,57,58,58,59,60,61,63,64,66,68,70,72,74,76,78,81,83,85,87,89,90,92,
93,94,95,96,96,96,96,95,95,94,93,91,90,88,86,84,82,80,77,75,73,71,69,67,65,63,62,61,60,
59,58,58,58,58,58,58,59,60,61,62,63,64,66,67,68,70,71,72,73,74,75,75,75,75,75,75,74,73,72,
71,69,67,65,63,60,58,55,53,50,47,44,42,39,36,34,32,30,28,26,25,24,23,22,22,21,21,22,22,
23,24,25,26,27,28,30,31,32,33,35,36,37,38,38,39,39,39,39,38,38,37,36,34,33,31,29,27,25,
23,21,19,17,14,12,10,8,7,5,4,2,1,1,0,0,0,0,1,2,3,4,6,7,9,11,13,15,17,20,22,24,26,28,30,32,
33,34,36,37,37,38,38,38,38,38,37,36,35,34,33,32,31,29,28,27,25,24,23,22,21,21,20,20,20,
21,21,22,23,24,26,28,30,32,34,37,39,42,44,47,

//SinF1L32SinF6L16
48,51,54,57,60,63,65,67,69,71,72,72,73,73,73,72,71,70,69,67,65,64,62,60,59,57,56,55,54,54,
54,54,55,56,57,58,60,62,65,67,70,73,75,78,81,84,86,88,90,92,93,94,95,95,95,94,93,92,91,
89,87,85,82,80,78,75,73,71,69,67,66,65,64,63,63,63,64,65,66,67,69,70,72,74,76,78,80,82,
84,85,86,87,88,88,88,87,86,85,83,81,79,77,74,71,68,65,62,59,56,53,51,48,46,45,43,42,41,41,
41,41,42,43,44,45,46,48,50,51,53,54,55,57,57,58,58,58,58,57,56,54,53,50,48,45,43,40,37,
34,31,28,25,22,19,17,15,13,12,11,11,10,10,11,12,13,14,16,18,19,21,23,25,27,29,30,32,33,
33,34,34,34,33,32,31,29,27,25,23,21,19,16,14,11,9,7,5,4,2,1,1,1,1,1,2,3,5,7,9,11,14,16,19,
22,24,27,30,32,34,36,37,38,39,40,40,40,39,38,37,36,35,33,31,30,28,26,25,23,22,21,20,20,
20,21,21,22,24,26,28,30,33,36,39,42,45,48,51,54,

//SinF1L32SinF7L16
48,52,55,58,61,64,67,69,70,71,72,72,71,70,69,68,66,64,62,60,58,56,54,53,52,51,51,52,53,54,
56,58,60,63,66,69,73,76,79,82,85,87,89,91,92,93,93,92,91,90,88,86,84,82,79,76,74,72,69,
68,66,65,64,64,64,65,66,68,70,72,74,77,79,82,85,87,89,91,92,93,93,93,93,92,90,88,86,83,
80,77,74,70,67,64,61,59,57,55,54,53,53,53,53,54,55,57,59,61,63,65,67,69,71,72,73,73,73,73,
72,70,68,66,63,60,57,53,50,46,43,39,36,33,31,29,28,27,26,26,26,27,28,30,32,34,36,38,40,
42,43,44,45,46,46,46,45,43,42,39,37,34,31,28,24,21,18,15,12,10,8,6,5,4,4,4,5,6,8,10,12,
15,17,20,22,25,27,29,30,31,32,32,32,31,30,28,26,24,22,19,16,14,11,9,7,5,4,3,2,2,3,4,5,7,
9,12,15,18,21,25,28,31,34,36,38,40,41,42,42,42,42,41,39,38,36,34,32,29,27,26,24,23,22,21,
21,22,23,24,26,29,31,35,38,41,45,48,52,55,

//SinF1L32SinF8L16
48,52,56,59,62,65,67,69,70,71,71,70,69,67,65,63,60,58,56,53,52,50,50,49,50,51,52,54,57,60,
63,67,71,74,78,81,84,87,88,90,91,91,90,89,88,85,83,80,78,75,72,69,67,66,64,64,63,64,65,
66,69,71,74,77,80,83,86,89,91,93,94,95,95,95,94,92,90,87,84,81,78,74,71,68,65,63,61,59,
59,58,59,60,61,63,66,68,71,73,76,78,80,81,82,82,82,81,79,77,74,71,68,64,60,56,53,49,46,43,
41,39,38,38,38,39,40,41,43,46,48,50,53,55,56,57,58,58,58,57,55,53,50,47,43,40,36,32,28,
25,22,19,17,15,14,14,14,15,16,18,20,23,25,28,30,33,35,36,37,38,37,37,35,33,31,28,25,22,
18,15,12,9,6,4,2,1,1,1,2,3,5,7,10,13,16,19,22,25,27,30,31,32,33,32,32,30,29,27,24,21,18,
16,13,11,8,7,6,5,5,6,8,9,12,15,18,22,25,29,33,36,39,42,44,45,46,47,46,46,44,43,40,38,36,
33,31,29,27,26,25,25,26,27,29,31,34,37,40,44,
}; 

//env and timer:
byte T0Div=0; //divider for T0 interrupt handling (to get high speed PWM and low speed handling from the same timer)
boolean Tick=false; //timeflag to start update modulation values and read potmeters 
byte TickDiv=0; //byte TickDiv=0;//allows F-PWM from timer0 to be 4x F-tick
byte TickCnt=0; // counter used to make tempo-related delays
// table for PWM rate, not synced on Tick, no reset on keystart:
int Rtable[35] ={2,3,4,5,6,8,10,12, 15,20,25,32,45,60,80,100, 125,150,200,250,320,450,600,800, 1000,1250,1500,2000,2500,3000,3500,4000 ,4500,5000,6000};
//int Rtable[35] ={2,3,4,5,6,8,10,12, 16,20,24,32,48,64,80,96, 128,160,196,256,384,512,768,1024, 1280,1536,2048,2304,2500,3072,4096,4608 ,5120,6144,8192};
// table for all other rates, synced on Tick
//int Tktab[32] ={4,6,9,12,16,18,24,32,36,42,48,60,72,84,96,108,128,144,168,192,240,288,336,384,432,480,576,768,960,1152,1536,2304};
int Tktab[64] ={4,5,6,7,8,9,10,11, 12,14,16,18,20,22,24,26, 28,30,32,34,36,38,40,42, 48,54,60,66,72,78,84,90,
96,102,108,112,118,128,138,144, 154,168,180,192,212,240,260,288, 300,336,384,432,460,480,576,650, 768,850,960,1152,1300,1536,1800,2304};
//SubOsc, variables:
byte Sub=0;// subOsc signal
byte SubWav=0; // bitpattern for sub output wave
byte SubLoop=0;
byte SubAmp=0;//subOsc dynamic amplitude (0..31)
byte SubLvl=0;//subosc potm level setting 
byte SubAM=255;//SubOsc output AM modulator (1=signal through, =0 off)
boolean SubModeAM=0;//1=SubOsc is in AM mode
byte SubMode=0; // 0=off, 1=level, 2= env, 3=filter, 4=vca1, 5=vca2, 6=digital AM
boolean SubFlm=false;//subosc filtermod mode
byte SubFlvl=0;//level of filtermodulation
byte SubDF=0; //Subosc digital lowpassfilter register
word SNoiseCnt=0; //SubOsc Noisepattern counter
word Nstate =1; // Pseudo Random Bit Sequence seed value for SubOsc noisegen
byte SubLen=8; // total waveform cycle length
byte SubPls=4; // total logic high length in cycle 
byte SubPnt=0; // pointer in sub waveform cycle
boolean SubSync=false;//flag to sync subwave to osc (sub formantmode)
//byte SubBit=0; // Sub output bit
byte SubDivtab [8] = {127,63,31,15,7,3,1,0};//oct divider wavemode 2
byte SubDiv=0;//freq divider by 8 for wavemode 2
byte SubDivOct;//freq div for wavemode 3,4 divide by oct
//byte SubNlen[88]PROGMEM ={255,250,245,240,235,230,225,221, 218,216,214,212,210,208,206,204, 202,200,198,196,194,192,190,188,
//187,186,185,184,183,182,181,180, 179,178,177,176,175,174,173,172, 171,170,169,168,167,166,165,164, 163,162,161,160,159,158,157,156,
//155,154,153,152,148,133,128,120, 116,110,104,98,88,82,78,74, 66,64,58,54,48,46,40,38, 36,32,30,28,23,19,18,15};
byte SubNlen[64]PROGMEM ={255,230,220,210,200,190,185,181, 180,179,178,177,176,175,174,173, 172,171,170,169,168,167,166,165, 
164,163,162,161,160,159,158, 157,156,155,154,153,152,148,133, 128,120,116,110,104,98,88,82, 78,74,66,64,58,54,48,46, 
40,38,36,32,26,24,23,19};
byte SubSkip[24] PROGMEM={1,2,3,4,5,6,7,8, 9,10,11,12,13,14,15,16, 18,20,22,24,26,28,30,32};
// end of sub

byte Ledtab[32]={1,30,50,65,80,100,120,140, 160,175,190,200,207,214,220,225, 
                 230,233,236,238,240,242,244,245, 246,247,248,249,250,251,252,253};// intensity table LED-PWM
byte LedPWMcnt=0;//LED PWM counter

//Envelope1: (Modulator 1) 
boolean E1=false;
boolean E1Loop=false;// single shot or loop mode
word E1TC=0;//TickCounter for E1
word E1Len=0;//calculated length, in ticks
byte Rn1=1; // random seed for pattern
byte LAST_Rn1=0;// last random number 
byte Rn1_F=0; //M1 noise filter
word E1cnt=0;// E1 value counter (phase in cycle)
word E1cntInc=0;//E1 counter increment
byte E1val=0;
byte E1Sval=0;//Osc shape modulation from modulator 1
byte E1Fval=0; //FilterF mod value from modulator 1
byte E1Aval=255;//VCA1 mod from modulator 1
byte E1Ccnt=0;//cycle counter
boolean E1Accent=0;//pattern function: accent yes/no
byte E1TrP=0;// E1 trigger pattern
byte BitSel=0;
byte E1pw=2;//sets pulse width for pulse shape
//Envelope2:
boolean E2=false;
boolean E2Loop=false;
word E2TC =0; //TickCounter for E2
word E2Len=0; //calculated length, in ticks
word E2cnt=0; // E2 value counter (phase in cycle)
byte E2Ccnt=0;//cycle counter
word E2cntInc=0; // E2 counter increment
byte E2val=0;   // E2 value calculated from E2cnt and shape
byte E2Fval=0;  // Filter control amount from E2val
//byte E2F1val=0;  // software Filter control amount from E2val
byte E2Aval=255; //VCA2 modulation amount
byte E2sw=0; //subosc wavemodulation
char E2Pval=0;  // Pitch control amount from E2val
byte Rn2=1; // E2 random seed, actual value
byte LAST_Rn2=0;// last random number
byte Rn2cnt=0; // E2 Random number counter
byte Rn2_F=0; //M1 noise filter
//Envelope0: PWM on Wavepotmeter:
boolean E0=false;
word E0cnt=0;
boolean E0dir=0;// direction bit for making smooth triangular PWM wave
byte E0val=0;
byte E0inc=0;
//Envelope0: vib on Wavemodpotmeter:
boolean E3=false;
word E3cnt=0;
boolean E3dir=0;// direction bit for making smooth triangular PWM wave
byte E3val=0;
byte E3inc=0;
int OscFM=0;
// storing synth sounds, setups:
word addr=0;//adrespointer for memory location soundnr
//byte EncRead=0;
//byte LAST_EncRead=0;
//char EncVal=0;
//byte EncAvg=127;
byte KbRead=0;//keyboard reading
byte LAST_KbRead=0;
byte KbVal=0;//keyboard note value
byte LAST_KbVal=0;//keyboard previous note value
byte KbShft=24; //keyboard note shift
//byte ParVal=0; //value of parameter that is currently edited 
//byte ParNr=0; //keynumber of parameter key that is currently selected 36..56 
//byte LAST_ParNr=0;
//byte ParRep=0;//repeat counter for parameter nr
//boolean KbDown=false;//note key not pressed
//boolean KpDown=false;//parameter key not pressed
//byte KpMode=0; //0=play node, 1=select load location, 2 select save location
byte PotmBp=36;//general beatperiod for all timing

//software SV filter
int LP = 0;
int HP =0;
//byte Q1 =0;//fixed Q
int BP = 0;
byte F1 =0;
int OmVal=0;
//control voltage  LDR
byte CvMode=1; //0=Key, 1=VCF, 2= KEYtrig + VCF
byte CvRead=127;//8 bits reading of control voltage
byte LAST_CvRead=127;
byte CvMidscale=127;
byte CvFVal=0;
byte CvKVal=32;
byte CvAVal=127;

void setup(){
 pinMode(2, OUTPUT); //  LED OSC main
 pinMode(3, OUTPUT); //  TIM2 OCR2B >> voltage VCA, LED Level
 pinMode(4,OUTPUT);   //>> LED Modulator2
 pinMode(5,OUTPUT);   //TIM0 OCR0B, >>Filter-Q, LED Q
 pinMode(6,OUTPUT);   //TIM0 OCR0A,  >>Filter-F, LED F
 pinMode(7, OUTPUT); // LED OSC sub
 pinMode(8,INPUT_PULLUP); // PushButton SW2, KEY
 pinMode(9,INPUT_PULLUP); // PushButton SW3, program-
 pinMode(10, OUTPUT);   //TIM1 OSC pulse output
 pinMode(11, INPUT_PULLUP);// PushButton SW1, program+
 pinMode(12, OUTPUT);//LED Modulator1 
 pinMode(13, OUTPUT);// Filter bit HP/LP
 pinMode(A5, INPUT);// analog in LDR bridge
DDRC=B00001110;
//init Timer 1, oscillator runs at Fkey for pulse or 8xFkey for wave/pat/noise --------------
TCCR1A=B00100001;   //COM1A1/COM1A0/COM1B1/COM1B0/-/-/WGM11/WGM10
TCCR1B=B00010010;   // x/x/x/WGM13/WGM12/CS12/CS11/CS10
//CSxxx:000=stop,001=div1,010=div8,011=div64,100=div256
TIMSK1=B00000010; // Interrupt on: OCRA,(OCRB,OCRA,OVERFLOW)
OCR1A = 0;  
// initialize Timer2 mode register for 2 channel PWM controlling FilterF and VCA:--------------
TIMSK2=B00000000;//no interrupts
TCCR2A=B10100001;//pwm phase correct
TCCR2B=B00000001;//prescaler=1, pwm=31kHz
//init Timer 0 for fixed freq update "Tick", fixed freq "SubOsc waves" and 2x PWM Filtermode
TIMSK0=B00000000;//no interrupts 
TCCR0A=B10100011;//COM0A1/COM0A0/COM0B1/COM0B0/-/-/WGM01/WGM00
TCCR0B=B00000001;// FOC0A/FOC0B/x/x/WGM02/CS02/CS01/CS00
//CSxxx:000=stop,001=div1,010=div8,011=div64,100=div256
// 111=/1024, 110=/256, 101=/128, 100=/64. 011=/32, 010=/8, 001=/1, 000=stop
TIMSK0 |= (1 << OCIE0A);// enable Timer0 compare interrupt
Serial.begin(31250); // MIDI Baudrate
//Serial.begin(38400); // debugging Baudrate
addr=0;//set patchnr to 0
//RecallPatch();//recall patch0 from EEPROM 
OCR2B=150;//VCA1 on
// read power-up state of Cv to determine CvMode
CvRead=(analogRead(5)>>2);//read control voltage, 8 bits analog
if (CvRead <48) {CvMode=2;}//right LDR blocked
else if (CvRead >208) {CvMode=0;}//left LDR blocked
else {//no LDR blocked, calibrate midscale value
CvMode=1;
CvMidscale=CvRead;
}
// end of read power-up state Cv 
}

 void loop() {
 MIDI_Poll(); //check for MIDIdata
//set timing tick for modulators:
  if (MidiClk==false){ // check for Midi clocksignal, if not: run on internal tick
   if (TickDiv > 37){ // update potmeters and modulators every 10ms=37
      Tick=true;
      TickDiv=0;
   }

  }

else if (Mclktim > 10000){MidiClk=false;}//timeout for Midi clocksignal, reset to internal tick
if (Tick == true){//update modulators and read potm-parameters each tick
/*
// Keyboard readout handling --------------------------------------------------
//if (MIDIcon==false){//check if local control is enabled, otherwise MIDIcontrol
KbRead=analogRead(7)>>2 ;//read keyboard
if (KbRead ==LAST_KbRead){//stability check, if stable do processing
KbVal=KbRead>>2 ;
if (KbVal !=LAST_KbVal){//if new keyboard value
if (KbVal > 10 ){//if keyboard active, 0..10 means no key pressed
  if (KbVal < 36){//if keyboard note keys pressed, KbVal 12...35
  switch (KpMode){ //0=play note, 1=select prog load location, 2=select prog save location
  case 0:
    KEY_DOWN=true;
        LAST_VEL = 83;
        Note=KbShft+ KbVal;  
        //Note=KbVal; 
        if (KbDown == false){//key pressed first time, do trigger
          KEY_START=true;
          GLIDE_NOTE=LAST_NOTE;
          LAST_NOTE=Note;
          KbDown=true;
       }
    else{
    LAST_NOTE=Note;//note change, no retrigger
    }
  break;
  case 1:
    ParNr=39;
    LAST_PROG=KbVal-12;
    RecallPatch();
    KpMode=0;
  break;
  case 2:
    ParNr=39;
    LAST_PROG=KbVal-12;
    StorePatch();
    KpMode=0;
  break;
   }
  }
   else{//then parameter key is pressed
     KpMode=0;
     KpDown=true;
     ParNr=KbVal;
     if (ParNr==LAST_ParNr){ParRep++;}//if same parameter key pressed again update repeat counter
     else {ParRep=0;}// else reset repeat counter
     LAST_ParNr=ParNr;
     LV1=0;
     LV2=0;
   }
 }
    else{ //if no key pressed
      KpMode=0;
      KbDown=false;
      KpDown=false;
      KEY_DOWN=false;
    }
   }
  LAST_KbVal=KbVal; 
  }
LAST_KbRead=KbRead;
// end of keyboard readout handling --------------------------------
*/
// Cv handling-----------------------------------------------
CvRead=(analogRead(5)>>2);//read control voltage, 8 bits analog 
switch(CvMode){
  case 0:
    CvFVal=CvRead;//read control voltage, 8 bits analog 
    CvKVal=32;
    //CvAVal=CvMidscale;
  break;
  case 1:
    //if (CvRead >(CvMidscale+16) && PB1==0){//LDR- changes Note if PB1 is pressed todo LDR=note LDR+=amp
    if (PB1==0){//LDR- changes Note if PB1 is pressed
    Note=(CvMidscale>>1)-(CvRead>>2);//read control voltage, 6 bits analog 
    PotmKm=8;
    GLIDE_NOTE=LAST_NOTE;
    LAST_NOTE=Note;
    CvFVal=CvMidscale;
    //CvAVal=CvMidscale;
    }
   
   // else if (CvRead <(CvMidscale-16) && PB1==0){//LDR- changes Note if PB1 is pressed todo LDR=note LDR+=amp
   // CvFVal=CvMidscale;
   // CvAVal=CvMidscale-CvRead;
   // }
     
     else if (CvRead>(LAST_CvRead+6)){
      KEY_DOWN=false;
      }
      else if (CvRead<(LAST_CvRead-6)){
      KEY_DOWN=true;
     KEY_START=true;
      }
      else{CvFVal=CvRead;}
  break;
  case 2:
    if (CvRead>(CvMidscale+32)){
    KEY_DOWN=true;
    if (CvRead>(LAST_CvRead+4)){KEY_START=true;}
    }
    else if (CvRead <(CvMidscale-16)) {CvFVal=CvRead;}
    else {CvFVal=CvMidscale;}
    CvKVal=32;
  break;
}
LAST_CvRead=CvRead;
//}
// end of Cv handling ---------------------------------------
// pushbutton handling:
PB1=digitalRead(8);//PB key
if (PB1==0){
///if (LAST_PB1==1 && KEY_DOWN==false){KEY_START=true;}//reset for LDR keydown
if (LAST_PB1==1){KEY_START=true;}
KEY_DOWN=true;
}
if (PB1==1 && LAST_PB1==0){KEY_DOWN=false;}
LAST_PB1=PB1;
//
PB2=digitalRead(11);//PB program+
if (PB2==0){
  if (LAST_PB2==1){
    if (LAST_PROG <23){LAST_PROG++;}
    RecallPatch();
  }
}
LAST_PB2=PB2;
//
PB3=digitalRead(9);//PB program-
if (PB3==0){
  if (LAST_PB3==1){
    if (LAST_PROG >0){LAST_PROG--;}
    RecallPatch();
  }
}
LAST_PB3=PB3;

// end of pushbutton handling --------------------------------
// key handling
if (KEY_START==true){ // a new key:init envelopes
 if (ampEnv<80){
 WavP=0;
 SubSync=true;
 }
 if (SubMode==1){SubLvl=8;}
 if( PotmOm > 63){
  E0cnt=0;
  E0dir=0;
  E0=false;
 }
if (PotmM1p <64 || PotmM1r <64){ // reset modulator 1
     E1=true;
     E1TC=0;
     E1Ccnt=0;
     E1Fval=0;
     E1cnt=0;
     Rn1=LAST_VEL;
     BitSel=0;
}
 if (PotmM2p <64 || PotmM2r <64){ // reset modulator 2
     E2=true;
     E2TC=0;
     E2Ccnt=0;
     E2cnt=0;
     Rn2cnt=0;
     E2Pval=0;
     E2Fval=0;
 }
    ampEnv=128;
    ampDcy=0;
    KEY_STARTcnt=0;//reset counter from keystart on, used for delayed vibrato
 }


// Amplifier settings: ----------------------------------------------------------------------------
// VCA handling:
//if (KEY_STARTcnt==0){ampEnv=64;}
//else if (KEY_STARTcnt==1){ampEnv=128;}
if (KEY_DOWN ==true){
   if (ampEnv>0 && PotmAm !=0 && PotmAm<65){// env mode is "Decay"
      ampDcy++;
      if (ampDcy>(64-PotmAm)){
        ampEnv=ampEnv-1;
        ampDcy=0;
      }
   }
}//end of KEY_DOWN is true
  else{//on key-up, ramp amp down
   if (PotmAm <123){//if env not in hold-mode
     if (ampEnv>0 && PotmAm>64){//if env mode is "Release"
       ampDcy++;
       if (ampDcy>(PotmAm-64)){
         ampEnv=ampEnv-1;
         ampDcy=0;
       }
     }
 else{
   if (ampEnv >3){ampEnv=ampEnv-4;} 
   else {ampEnv=0;} // or use code below to mimic reverb
//    if (ampEnv>80){ampEnv=ampEnv-3;}//normal decay
//    else{ //artifical reverb by long decay at low level
//         ampDcy++;
//         if (ampDcy>4 && ampEnv>0){
//         ampEnv=ampEnv-1;
//         ampDcy=0;
//        }
//      }
     } 

  }
}
//VCA=constrain(((E2Aval*PotmAv2)/128)-oct-(PotmAv1>>1)+32-QAtab[PotmFm/4],0,255);
//VCA=constrain(((127-CvAVal)+(E1Aval*PotmAv2)/128)-oct,0,255);
VCA2=constrain(((E1Aval*PotmAv2)/128)-pgm_read_byte_near(VCAkey+LAST_NOTE),0,255);
//VCA2=constrain(PotmAv2-oct,0,255);
OCR2B=pgm_read_byte_near(VCAtab + VCA2*ampEnv/256);
//end of amp settings ------------------------------------------------------------
// PotmKm handling,  controls PotmKvib and PotmKprt
if (PotmKm != LAST_PotmKm){
if (PotmKm <3){
PotmKvib=0;
PotmKprt=PotmKm;
}
else if(PotmKm <64){
PotmKprt=2;
PotmKvib=PotmKm-3;
} 
else{
PotmKprt=0;
PotmKvib=PotmKm-64;
} 
LAST_PotmKm=PotmKm;
}
// end of PotmKm handling
//note settings -------------------------------------------------------------
if (PotmKprt >0){
 GlideTm=GlideTm+2;
   if (GlideTm > PotmKprt){
     if (GLIDE_NOTE < LAST_NOTE){GLIDE_NOTE++;}
     else if (GLIDE_NOTE > LAST_NOTE){GLIDE_NOTE--;}
     GlideTm=0;
   }
  Note=(32-CvKVal)+GLIDE_NOTE+(LAST_PWH/8)-8+E2Pval/6;
 }
else {Note=(32-CvKVal) +LAST_NOTE+(LAST_PWH/8)-8+E2Pval/6;}
Note_to_Freq(Note);// call subroutine to calculate note
//end of note settings ----------------------------------------------------

// Osc handling ----------------------------------------------------------
//PotmOwave:0=off,1-31=pulse,32-64=PWM,65Sine,66-92=Saw,93-119=Pattern,120-127=Noise
//WavMode 0=off, 1=pulse-pwm, 2=wavetable, 3=pattern, 4=noise-BR, 5=noise-LP, 6=noise-BP, 7=noise-HP 
//PotmOw=PotmOwP+WavPmod;
if (PotmOw != LAST_PotmOw){
  LAST_PotmOw=PotmOw;
//   if (PotmOw==0){// osc off
//   DDRB=DDRB&B11111011;//pinMode(10,INPUT);
//   PW=0;
//   WavMode=0;

//   }
     if (PotmOw <64){
     DAC=128;
     WavMode=PotmOw>>2; // disable oscillator pattern-noise output
     if (WavMode>0){
     DDRB=DDRB | B00000100;// pinMode(10,OUTPUT);//enable pulse output
     TCCR1B=B00010010;// x/x/x/WGM13/WGM12/CS12/CS11/CS10, set prescaler tim1 to 8
     }
    else {DDRB=DDRB&B11111011;}//pinMode(10,INPUT);
   }
   else if (PotmOw >63){
    DDRB=DDRB&B11111011;//pinMode(10,INPUT);// disable pulse output
    TCCR1B=B00010001;//set prescaler tim1 to 1 
         if (PotmOw  <80){
         WavMode=16;// enable wavetable readout
         }
        else if (PotmOw <120 ){
         WavMode=28;//enable looped noisepattern out
         LP=0;
         BP=0;
         HP=0;
         NoiseLen=NoiseLentab[(PotmOw-80)];
        } 
        else if (PotmOw <122){
         WavMode=29; // enable noise out
        } 
        else if (PotmOw <124){
         WavMode=30; // enable noise out
        } 
        else if (PotmOw <126){
         WavMode=31; // enable noise out
        } 
        else {
         WavMode=32; // enable noise out
        } 
   }
 }
WavSel=(PotmOw-64)<<8; // <<8 takes 256 steps is 1 waveform, smaller steps combine waveforms
if (WavSel >(17<<8)){WavSel=WavSel-(17<<8);}
// End of Osc handling ----------------------------------------------------------------

//SubOsc settings------------------------------------------------------------------------
 //  if (SubMode==1){//Sub to audio, level
//    SubLvl=SubLvl*48/(16+PotmSm);
//   }
if (PotmSm != LAST_PotmSm){//sub to audio out, set static level
  if (PotmSm <32){ //set SubOsc level
    SubMode=0;
    SubModeAM=false;
    SubAM=B11111111;
   SubFlm=false;
    SubLvl=PotmSm<<1;
  }
  else if (PotmSm <64){ //sub to audio out, level ramp down
  SubMode=1;
  SubModeAM=false;
  SubAM=B11111111;
 SubFlm=false;
  }
else  if (PotmSm <120){//sub to VCF freq mod
  SubMode=2;
  SubModeAM=false;
  SubAM=B11111111;
  SubFlvl=PotmSm-64;
  if (SubFlvl>filterF){SubFlvl=filterF;}
  SubFlm=true;
   SubLvl=0;
  // if (WavMode<16){DDRB=DDRB | B00000100;}//pinMode(10,OUTPUT);
  }
  else {//sub to AM mod
   SubMode=3;
    SubModeAM=true;
    SubFlm=false;
    SubLvl=0;
  }  
LAST_PotmSm=PotmSm;
SubAmp=SubLvl;
}
if ((PotmSw != LAST_PotmSw) || PotmM2d==127){
PotmSw=PotmSw+E2sw;
  if (PotmSw<88 && PotmSw>64)
    {
     SubLen=107-PotmSw;
     //todo
    }
   else{
   //SubLen=pgm_read_byte_near(SubWavPerTab+127-PotmSw);//in slow loop
  // SubWav=pgm_read_byte_near(SubWavTab+127-PotmSw);//in slow loop
 SubLen=129-PotmSw;
 SubPls=SubLen/2;
   }
 LAST_PotmSw=PotmSw;
}
//End off SubOsc settings------------------------------------------------------------------

//E0 modulator handling, dedicated LFO for PWM and Oscwaveshape ---------------------------
 if (PotmOm <33){
  PW=E1Sval+2+(PotmOm)<<2;
  E0cnt=0; // reset PWM LFO to 50%
  E0dir=0;
  }
else if (PotmOm > 32 && PotmOm <51){ // if PotmOw is set for PWM
    E0inc=(PotmOm-32) +LAST_NOTE/8;  // PWM enabled, set speed, increase with key
    if (E0inc>34) {E0inc=34;}
    {E0cnt=E0cnt+Rtable[E0inc];}
    if (E0cnt > 32640){E0cnt=32640;}
    //triangular wave:
       switch (E0dir){
       case 0: 
        E0val=255-(E0cnt>>7);
        if (E0cnt==32640){E0dir = 1;}
       break;
       case 1: 
       E0val=E0cnt>>7;
       if (E0cnt==32640){E0dir = 0;}
       break;
       }
 if (E0cnt==32640){E0cnt=0;}
 PW =E1Sval+ 4+(E0val>>1);
}
else if(PotmOm > 50 && PotmOm <64){
E0inc=(48-32) +LAST_NOTE/8;  // PWM enabled, set speed, increase with key
if (E0inc>34) {E0inc=34;}
{E0cnt=E0cnt+Rtable[E0inc];}
if (E0cnt > 32640){E0cnt=32640;}
//triangular wave:
       switch (E0dir){
       case 0: 
        E0val=255-(E0cnt>>7);
        if (E0cnt==32640){E0dir = 1;}
       break;
       case 1: 
       E0val=E0cnt>>7;
       if (E0cnt==32640){E0dir = 0;}
       break;
       }
 if (E0cnt==32640){E0cnt=0;}
 PW =E1Sval+4+4*(PotmOm-50)+E0val/(PotmOm-49);
}
else if (PotmOm > 63 && PotmOm <84){ // if PotmOw is set for PWM keysync with fast attack
   if (E0==false) {E0inc=(PotmOm-57) +LAST_NOTE/8;}  // PWM high speed attack, increase with key
   else{E0inc=(PotmOm-63) +LAST_NOTE/8;} // PWM normal speed lfo, increase with key
    if (E0inc>34) {E0inc=34;}
    {E0cnt=E0cnt+Rtable[E0inc];}
    if (E0cnt > 32640){E0cnt=32640;}
    //triangular wave:
       switch (E0dir){
       case 0: 
        E0val=255-(E0cnt>>7);
        if (E0cnt==32640){E0dir = 1;}
       break;
       case 1: 
       E0=true;
       E0val=E0cnt>>7;
       if (E0cnt==32640){E0dir = 0;}
       break;
       }
 if (E0cnt==32640){E0cnt=0;}
 PW =E1Sval+4+(E0val>>1);
}
//if (WavMode<16){
//PW=E1Sval+(PW*WavMode)/8;  
//}
//else if (WavMode<16){
//PW=E1Sval+(PW*(WavMode-8))/16;  
//}

// end of E0 handling ----------------------------------------------------------------

//E3 modulator handling, dedicated LFO for OscFM -------------------------------------
{E3cnt=E3cnt+Rtable[27];}
if (E3cnt > 32640){E3cnt=32640;}
//triangular wave:
       switch (E3dir){
       case 0: 
        E3val=255-E3cnt>>7;
        if (E3cnt==32640){E3dir = 1;}
       break;
       case 1: 
       E3val=E3cnt>>7;
       if (E3cnt==32640){E3dir = 0;}
       break;
       }
 if (E3cnt==32640){E3cnt=0;}
if (PotmKvib >0){//delayed vibrato
 OscFM = (E3val-127)*(LAST_AFT+PotmKvib*KEY_STARTcnt>>8)>>2;  
}
else {OscFM = (E3val-127)*(LAST_AFT)>>2;}
// end of E3 handling -------------------------------------------------------------------

// Modulator E1 handling: PotmM1r=rate PotmM1d=depth PotmM1p=accent pattern --------------
if (E1==true || PotmM1r>63){//if envelope still runs or when in lfo-mode 
  if (KEY_START==true || (PotmM1r != LAST_PotmM1r)){ //update parameters on new key or potm change
  LAST_PotmM1r=PotmM1r;
   if (PotmM1r <64){ // if Envelope mode
     E1Loop=false; // disable waveform looping
     E1Len=Tktab[(63-PotmM1r)];
     E1pw=2;//PotmM1r-93;
     E1cntInc=4892/E1Len; // calculate counter increment
  }
    if (PotmM1r >64) { // LFO mode
       E1Loop=true; // enable waveform looping for LFO mode
       E1Len=Tktab[(127-PotmM1r)];// potmeter sets reprate, calculate waveform length in ticks
       E1cntInc=4892/E1Len; // calculate counter increment
     }
//  else if (PotmM1r==64){E1val=0;}//{E1val=E1Aval+E1Fval;}
   }
   //patterntrig:

if (PotmM1p<64) {E1TrP=255-PotmM1p;} //potm sets pattern Trigger Pattern byte setting
else {E1TrP=128+PotmM1p;}
  if (E1TC==0){     // if start mod1 wave cycle
    Rn1=(Rn1<<1 | (((Rn1>>7)&1) ^ ((Rn1>>5)&1) ^ ((Rn1>>4)&1) ^ ((Rn1>>3)&1)) );// 8 bit PRBS
      if (PotmM1p > 63 && PotmM1p < 66){// potm centre: random pattern
        E1Accent = Rn1&1; // take bit 1 from the PRBS generator, seed is last velocity value
      }
      else{
      if (BitSel >7){BitSel=0;}  // set bitselector to 0 if >7
        E1Accent=(E1TrP>>BitSel)&1; // get a trigger-enable-bit from Trigger Pattern byte
        BitSel++; // take next bit
     }
  }

   switch (PotmM1s>>3) { //select waveform and calculate actual value
      case 0:// sawdown
      E1val=255-E1cnt/20;
      //E1val=255-E1cnt/(20-(PotmM1s>>1));
      break;
      case 1://sawup
      E1val=E1cnt/20;    
      //E1val=E1cnt/(20-((PotmM1s-16)>>1));
      break;
      case 2://saw-updown
      if (E1cnt <2446){E1val=E1cnt/10;}
      else{E1val=(4892-E1cnt)/10;}
      break;
      case 3://saw-downup
      if (E1cnt <2446){E1val=E1cnt/10;}
      else{E1val=(4892-E1cnt)/10;}
      E1val=255-E1val;
      break;
      case 4://square 50%
      if (E1cnt < 2446){E1val=255;}
      else {E1val=0;}
      break;
      case 5://square 50% inv
      if (E1cnt > 2446){E1val=255;}
      else {E1val=0;}
      break;
      case 6://square 25%
      if (E1cnt < 1223){E1val=255;}
      else {E1val=0;}
      break;
      case 7://pulse
      if (E1TC < 16){E1val=255;}// variable time pulsewidth
      else {E1val=0;}
      break;
      case 8://pulse
      if (E1TC < 8){E1val=255;}// variable time pulsewidth
      else {E1val=0;}
      break;
      case 9://pulse
      if (E1TC < 4){E1val=255;}// variable time pulsewidth
      else {E1val=0;}
      break;
      case 10://pulse
      if (E1TC < 2){E1val=255;}// variable time pulsewidth
      else {E1val=0;}
      break;
      case 11://random1
      if (PotmM1r <64){
      LAST_Rn1++;
        if (LAST_Rn1>= 32-PotmM1r/2){ 
         Rn1=(Rn1<<1 | (((Rn1>>7)&1) ^ ((Rn1>>5)&1) ^ ((Rn1>>4)&1) ^ ((Rn1>>3)&1)) );// 8 bit PRBS
         LAST_Rn1=0;
        }
      }
        Rn1_F= (Rn1_F)-(Rn1_F>>1)+(Rn1>>1);//lowpassfilter set by PotmM1s
        E1val= Rn1_F;
      break;
     case 12://random1
      if (PotmM1r <64){
      LAST_Rn1++;
        if (LAST_Rn1>= 32-PotmM1r/2){ 
         Rn1=(Rn1<<1 | (((Rn1>>7)&1) ^ ((Rn1>>5)&1) ^ ((Rn1>>4)&1) ^ ((Rn1>>3)&1)) );// 8 bit PRBS
         LAST_Rn1=0;
        }
      }
        Rn1_F= (Rn1_F)-(Rn1_F>>2)+(Rn1>>2);//lowpassfilter set by PotmM1s
        E1val= Rn1_F;
      break;
      case 13://random1
      if (PotmM1r <64){
      LAST_Rn1++;
        if (LAST_Rn1>= 32-PotmM1r/2){ 
         Rn1=(Rn1<<1 | (((Rn1>>7)&1) ^ ((Rn1>>5)&1) ^ ((Rn1>>4)&1) ^ ((Rn1>>3)&1)) );// 8 bit PRBS
         LAST_Rn1=0;
        }
      }
        Rn1_F= (Rn1_F)-(Rn1_F>>3)+(Rn1>>3);//lowpassfilter set by PotmM1s
        E1val= Rn1_F;
      break;
 
      case 14://pulsed noise
      if ((E1cnt>>8) < 16){
       Rn1=(Rn1<<1 | (((Rn1>>7)&1) ^ ((Rn1>>5)&1) ^ ((Rn1>>4)&1) ^ ((Rn1>>3)&1)) );// 8 bit PRBS variable time pulsewidth
       Rn1_F= (Rn1_F)-(Rn1_F>>2)+(Rn1>>2);
       E1val= Rn1_F;
      }
      else {E1val=0;}
      break;
     case 15://pulsed noise
      if ((E1cnt>>8) < 4){
       Rn1=(Rn1<<1 | (((Rn1>>7)&1) ^ ((Rn1>>5)&1) ^ ((Rn1>>4)&1) ^ ((Rn1>>3)&1)) );// 8 bit PRBS variable time pulsewidth
       Rn1_F= (Rn1_F)-(Rn1_F>>2)+(Rn1>>2);
       E1val= Rn1_F;
      }
      else {E1val=0;}
      break;
  }

 if (E1Accent ==0) {E1val=32+E1val*LAST_VEL/192;}
 E1Sval=0;
 //WavPmod=0;
    if (PotmM1d ==0){//fully left for VCA modulation
      E1Aval=E1val;//*(223+LAST_VEL/3)/256;
      SubAmp=SubLvl;
    }
   else if (PotmM1d < 64){  //Potmeter set for wavemod
  //WavPmod=(E1val*(64-PotmM1d))>>8;
   //WavPmod=0;
  //PW=PW+E1val*(64-PotmM1d)>>6;
  E1Sval=E1val*(64-PotmM1d)>>7;
  E1Aval=255;
  SubAmp=SubLvl;
  }
 else if (PotmM1d > 65){  //Potmeter rotated right for Filter modulation
   if (PotmM1d==127){//sub mod 
    E1Fval=0;
    E1Sval=0;
    //E1Aval=239+LAST_VEL/8;//no VCA amplitude modulation
    E1Aval=255;
    SubAmp=SubLvl*E1val/256;//Sub amplitude modulation
   }
   else{//filter mod
    SubAmp=SubLvl;
    E1Sval=0;
    //WavPmod=0;
    //E1Aval=239+LAST_VEL/8;
    E1Aval=255;
    if (ampEnv>4){
    E1Fval=(PotmM1d-65)*E1val/64;
    }
    else {E1Fval=0;}
     }
 }
  else if (PotmM1d==64) {// Potm is at mid: no modulation
  // WavPmod=0;
   E1val=0;
   E1Sval=0;
  // E1Aval=239+LAST_VEL/8;
   E1Aval=255;
   SubAmp=SubLvl;
  } 

 E1cnt=E1cnt+E1cntInc;
  if (E1cnt >4892){E1cnt=4892;}
  E1TC++;// increase E1 tick counter
  if (E1TC >= E1Len) {// if calculated length is reached
     E1TC=0;
     E1cnt=0;
     E1Ccnt++;
     if (PotmM1s !=8 && PotmM1s != 16){E1val=0;}// set modulator to zero if wave is not ramp up
     if (E1Loop==false){E1=false;}  // do not restart if in env mode
  }
}// End of E1 handling ------------------------------------------------------------------------

// Modulator E2 handling: PotmM2r=rate PotmM2d=depth PotmM2p=shapepattern ---------------------
if (E2==true || PotmM2r>63){//if envelope still runs or when in lfo-mode 
  if (KEY_START==true || (PotmM2r != LAST_PotmM2r)){ //update parameters on new key and potm change
      LAST_PotmM2r=PotmM2r;
    if (PotmM2r <64){ // Envelope mode
      E2Loop=false;
 
     E2Len=Tktab[(63-PotmM2r)];// potmeter sets rate
     E2cntInc=4892/E2Len;
    }
     else if (PotmM2r >64) { // LFO mode
       E2Loop=true;
       E2Len=Tktab[(127-PotmM2r)];
      E2cntInc=4892/E2Len;
    }
//  else if (PotmM2r==64){
//    E2val=0;
  //else{//PotmM2r=64
 // E2Loop=false;
//  E2Fval=0;
// E2Aval=255;
// E2Pval=0;
// }
 }//end of:update parameters on new key and potm change

     switch (PotmM2s>>3) { //select waveform and calculate actual value
      case 0:// sawdown
      E2val=255-E2cnt/20;
      break;
      case 1://sawup
      E2val=E2cnt/20;    
      break;
      case 2://saw-updown
      if (E2cnt <2446){E2val=E2cnt/10;}
      else{E2val=(4892-E2cnt)/10;}
      break;
      case 3://saw-downup
      if (E2cnt <2446){E2val=E2cnt/10;}
      else{E2val=(4892-E2cnt)/10;}
      E2val=255-E2val;
      break;
      case 4://square 50%
      if (E2cnt < 2446){E2val=255;}
      else {E2val=0;}
      break;
      case 5://square 50% inv
      if (E2cnt > 2446){E2val=255;}
      else {E2val=0;}
      break;
      case 6://square 25%
      if (E2cnt < 1223){E2val=255;}
      else {E2val=0;}
      break;
      case 7://pulse
      if (E2TC < 16){E2val=255;}// variable time pulsewidth
      else {E2val=0;}
      break;
      case 8://pulse
      if (E2TC < 8){E2val=255;}// variable time pulsewidth
      else {E2val=0;}
      break;
      case 9://pulse
      if (E2TC < 4){E2val=255;}// variable time pulsewidth
      else {E2val=0;}
      break;
      case 10://pulse
      if (E2TC < 2){E2val=255;}// variable time pulsewidth
      else {E2val=0;}
      break;
      case 11://random1
      if (PotmM2r <64){
      LAST_Rn2++;
        if (LAST_Rn2>= 32-PotmM2r/2){ 
         Rn2=(Rn2<<1 | (((Rn2>>7)&1) ^ ((Rn2>>5)&1) ^ ((Rn2>>4)&1) ^ ((Rn2>>3)&1)) );// 8 bit PRBS
         LAST_Rn2=0;
        }
      }
        Rn2_F= (Rn2_F)-(Rn2_F>>1)+(Rn2>>1);//lowpassfilter set by PotmM1s
        E2val= Rn2_F;
      break;

      case 12://random lpf2
     if (PotmM2r <64){
      LAST_Rn2++;
        if (LAST_Rn2>= 32-PotmM2r/2){ 
         Rn2=(Rn2<<1 | (((Rn2>>7)&1) ^ ((Rn2>>5)&1) ^ ((Rn2>>4)&1) ^ ((Rn2>>3)&1)) );// 8 bit PRBS
         LAST_Rn2=0;
        }
      }
        Rn2_F= (Rn2_F)-(Rn2_F>>3)+(Rn2>>3);//lowpassfilter 
        E2val= Rn2_F;
      break;
       case 13://sample Modulator 1
       if (E2cnt<1223){E2val= E1val;}
       else E2val=0;
      break;
  case 14://sample and decay Modulator 1
     if (E2TC==0){E2val= E1val;}
     else {
     if (E2val>1){E2val=E2val-2;}
     }
      break;
  case 15://sample and hold Modulator 1
     if (E2TC==0){E2val= E1val;}
     break;
 
  }

   if (E2TC == 0){//calculate new random,pattern number
        if (Rn2cnt ==0 && PotmM2p !=0 && PotmM2p !=127){Rn2=PotmM2p+1;}// potmeter sets pattern
      //? else if (Rn2cnt ==0 && PotmM2p ==0){if (E1TC ==0){Rn2++;}}// MWH sets pattern
        Rn2=(Rn2<<1 | (((Rn2>>7)&1) ^ ((Rn2>>5)&1) ^ ((Rn2>>4)&1) ^ ((Rn2>>3)&1)) );
        Rn2cnt++;
        if (Rn2cnt >7 && PotmM2p !=64){Rn2cnt=0;}// patternlength =8 or inf for ptm is centre
       // else if (Rn2cnt >15 && PotmM2p !=127){Rn2cnt=0;}// patternlength =16, or inf for potmeter full right 
     } 

  if (PotmM2p>0 && PotmM2p<127) {E2val=E2val*Rn2/255;}// if potm shapepattern>0 modulate envval with pattern 

   if (PotmM2d < 64){  //rotate left for Pitch modulation
     if (PotmM2d ==0) // 
     {
       E2Aval=E2val;
       //E2Pval=LAST_VEL*E2val/256;
       // E2OFMval=E2val;
       //to do
     }  
     else{
       E2Aval=255;
       E2Pval=(63-PotmM2d)*E2val/128;
     //E2OFMval=0;
     } 
 //E2F1val=0;
  }  
 else if (PotmM2d > 65){//rotate right for Filter modulation
 if (PotmM2d ==127){//fully right for M2>subwave value modulation
 // E2F1val=E2val;
  E2sw=(E2val>>3);
   E2Fval=0;
   E2Pval=0;
  }
 else{
   E2sw=0;
   if (ampEnv>4){
   E2Fval=(PotmM2d-65)*E2val/64;
    }
    else {E2Fval=0;}
   //E2Fval=(PotmM2d-65)*E2val/96;
   }
 E2Pval=0;
  }
else if (PotmM2d==64){
  E2val=0;
   E2Fval=0;
   E2Pval=0;

}
//  else E2val=0; ??
  E2cnt=E2cnt+E2cntInc;
  if (E2cnt >4892){E2cnt=4892;}
  E2TC++;// increase E2 tick counter
  if ((E2TC >= E2Len) || (PotmM2p >64 && E1cnt==0)) {// if calculated length is reached
 // if (PotmM2p==127 && E1TC==0) {// if calculated length is reached or E1 starts cycle
 // if (E2TC >= E2Len){
     E2TC=0;
     E2Ccnt++;
     E2cnt=0;
     if (PotmM2s !=8 && PotmM2s !=16){E2val=0;}// set modulator to zero if wave is not ramp up 16,48
     //    E2Pval=0;
     if (E2Loop==false){E2=false;}
  }
}// End of E2 handling --------------------------------------------------------------------------

//Filter settings, PotmFm sets Q, ModWheel+velocity+note+E1Fval+E2Fval sets F : ------------------
//filterF = 262- (PotmFf<<1)  - (LAST_NOTE) - E1Fval- E2Fval + CvFVal;//- LAST_VEL/4
filterF = 390- (PotmFf<<1)  - Note - E1Fval- E2Fval + CvFVal-CvMidscale;//- LAST_VEL/4
filterF = constrain(filterF, 0, 255); 
//if (SubFlm==false) {OCR0A=filterF;}//set PWM value for filterFrequency if SubOSC modulation is off
OCR0A=filterF;

if (PotmFm != LAST_PotmFm){ //Set filter mode if changed:
  
if (PotmFm <64){
OCR0B=PotmFm<<2;
PORTB = PORTB & B11011111;//digitalWrite (13,LOW);
}
else{
OCR0B=255-(PotmFm<<2);
PORTB = PORTB | B00100000;//digitalWrite (13,HIGH);
}  
   LAST_PotmFm=PotmFm;
 }
//end of Filter settings ----------------------------------------------------------------------

//software sv filter:
//OmVal=PotmOm+(E2F1val>>1)+(PotmOm>>1);
//OmVal= PW;//(E2F1val>>1)+PW;
F1= constrain(PW, 0, 150);
//Q1=16;
//end of software sv filter 
if (KEY_STARTcnt <255) {KEY_STARTcnt++;}
KEY_START=false;
 Tick = false;
 }//end of: if tick is true
}

 void MIDI_Poll(){
   if (Serial.available() > 0) {
   do {
       incomingByte = Serial.read(); // read the incoming byte:
       if (incomingByte>247) {       // system real-time messages
         if (incomingByte == 248) {  //a midiclock byte
         MidiClk=true; // enable MIDIclock sync
         MidiTick=true; // set MIDIclock event flag
         TickDiv=0;// reset timer
         }
       }
         else if (incomingByte>240) { //system exclusive data
          statusbuffer = 0;
         }
         else if (incomingByte>127) { //channel voice messages
           statusbuffer = incomingByte;
           firstbyte = true;
           statusByte = 0;
           dataByte = 0;
         }
         else if (statusbuffer!=0) {
         if (firstbyte == true) {
           // must be first byte
           statusByte = incomingByte;
           firstbyte = false;
            if (statusbuffer == PROG_CHANGE){ //program change
             LAST_PROG = statusByte;
             if (PB1==1){RecallPatch();}//read patch from eeprom if pb "key"not pressed
             else {StorePatch();}//if "key"pressed program patch in eeprom on this location
          }
           if (statusbuffer == AFT_CHANGE){LAST_AFT = statusByte;} //channel aftertouch
         } //end of: if first byte
         else { // it is second byte
           dataByte = incomingByte;//process the message
           if (statusbuffer == NOTE_ON && dataByte != 0) { //if MIDI note_on message with velocity >0
             //MIDIcon=true;//MIDI notes detected, stylus disabled
             KEY_DOWN = true;
             KEY_START =true;
             LAST_NOTE = statusByte;
             LAST_VEL = dataByte;
            }
             else if (statusbuffer == NOTE_OFF || (statusbuffer == NOTE_ON && dataByte == 0)) {
            //if MIDI note_off message or equivalent message: note_on with velocity=0
             if (statusByte == LAST_NOTE){
              KEY_DOWN = false;
              GLIDE_NOTE= LAST_NOTE;
             }
          }
           else if (statusbuffer == PITCHW_CHANGE){LAST_PWH = dataByte;} //pitch bend wheel                     
           else if (statusbuffer == CONT_CHANGE){  // other controllers:
           //todo replace by switch case?:
          //switch (statusByte){
        //case 1:
        //PotmFf = dataByte;
        //break;
        //} //end of switch case statusbyte
            if (statusByte==1) {PotmFf = dataByte;}//Modulation Wheel
            else if (statusByte==07) {PotmAv2= dataByte;}//channel volume
            else if (statusByte==10) {PotmM2p = dataByte;}//modulator 2 pattern
            else if (statusByte==71) {PotmM1p = dataByte;}//modulator 1 pattern
            else if (statusByte==72) {PotmM2d = dataByte;}//modulator 2 depth 
            else if (statusByte==73) {PotmFm = dataByte;}//potmeter R5 on UMX250 
            else if (statusByte==74) {PotmM1d = dataByte;}//potmeter R3 on UMX250 
            else if (statusByte==75) {PotmM2r = dataByte;}//modulator 2 rate 
            else if (statusByte==91) {PotmOw = dataByte;}//potmeter R1 on UMX250
            else if (statusByte==93) {PotmM1r = dataByte;}//potmeter R2 on UMX250             
            else if (statusByte==96) {PotmSm = dataByte;}// switch B2 SubOsc mode
            else if (statusByte==97) {PotmSw = dataByte;}// switch B1 data entry - 
            else if (statusByte==66) {PotmM1s = dataByte;}// switch B3 
            else if (statusByte==65) {PotmM2s = dataByte;}// switch B6        
            else if (statusByte==98) {PotmOm = dataByte;}// Oscillator mode, waveshape           
            else if (statusByte==90) {PotmKm = dataByte;}// Keyboard mode
            else if (statusByte==99) {PotmAm = dataByte;}// VCA env mode
          }
          //clear variables for next message:
          statusByte = 0;
          dataByte = 0;
          firstbyte = true;        
         }
       }
     }
  while (Serial.available() > 0); //{}
   }
}

void Note_to_Freq(byte Note){ // input MIDI note number, calculates timer1 division value, sets freq and pulse width
if (Note<12){Note=12;}
else if (Note>106){Note=106;}
    if (Note >95){
    oct=7;
    nt=Note-96;
    }
    else if (Note >83){
    oct=6;
    nt=Note-84;
    }
    else if (Note >71){
    oct=5;
    nt=Note-72;
    }
    else if (Note >59){
    oct=4;
    nt=Note-60;
    }
    else if (Note >47){
    oct=3;
    nt=Note-48;
    }
    else if (Note >35){
    oct=2;
    nt=Note-36;
    }  
    else if (Note>23){
    oct=1;
    nt=Note-24;
    }
   else {
    oct=0;
    nt=Note-12;
   }
if (WavMode <16 || WavMode >27){
  noInterrupts();
  OCR1A =(OscFM+Note_div_array[nt])>>oct;//-LAST_AFT*2; // set fine division value from array covering 1 octave, div by octaves
  //OCR1A = E2Pval*Note+Note_div_array[nt]/oct; // set fine division value(1-256) from array covering 2 octaves
  OCR1B = OCR1A/256*PW; // set correct pulse width
 // OCR1B = PW<<4;//fixed pulsewidth
   interrupts();
  }
else{
  noInterrupts();//to do use prescaler on tim1
  OCR1A =(OscFM+Note_div_array[nt])>>7;//-LAST_AFT*2; // set fine division value from array covering 1 octave, use skip/inc table
  //OCR1A = E2Pval*Note+Note_div_array[nt]/oct; // set fine division value(1-256) from array covering 2 octaves
  //OCR1B = OCR1A/256*PW; // set correct pulse width ?needed
  interrupts(); 
  }
    // Freq=16MHz/prescaler/(OCR2A+1) example:1381.2Hz for OCR2A=180 prescaler=64
    // Dutycycle=(OCRB+1)/(OCRA+1) example: 28.2% for OCRB=50 and OCR2A=180
    // prescaler word= CS22,CS21,CS20: 
    // 111=/1024, 110=/256, 101=/128, 100=/64. 011=/32, 010=/8, 001=/1, 000=stop
} // end of note to freq subroutine

void StorePatch(){
if (LAST_PROG >47){LAST_PROG=47;}
addr = 20*LAST_PROG;
EEPROM.write(addr, PotmM1r);
EEPROM.write(addr+1, PotmM1d);
EEPROM.write(addr+2, PotmM1s);
EEPROM.write(addr+3, PotmM1p);
EEPROM.write(addr+4, PotmM2r);
EEPROM.write(addr+5, PotmM2d);
EEPROM.write(addr+6, PotmM2s);
EEPROM.write(addr+7, PotmM2p);
EEPROM.write(addr+8, KbShft);
EEPROM.write(addr+9, PotmKprt);
EEPROM.write(addr+10, PotmKvib);
EEPROM.write(addr+11, PotmOw);
EEPROM.write(addr+12, PotmOm);
EEPROM.write(addr+13, PotmSw);
EEPROM.write(addr+14, PotmSm);
//EEPROM.write(addr+15, PotmAv1);
EEPROM.write(addr+16, PotmFf);
EEPROM.write(addr+17, PotmFm);
EEPROM.write(addr+18, PotmAv2);
EEPROM.write(addr+19, PotmAm);
}

void RecallPatch(){
if (LAST_PROG >47){LAST_PROG=47;}
addr = 20*LAST_PROG;
PotmM1r = EEPROM.read(addr);
PotmM1d = EEPROM.read(addr+1);
PotmM1s = EEPROM.read(addr+2);
PotmM1p = EEPROM.read(addr+3);
PotmM2r = EEPROM.read(addr+4);
PotmM2d = EEPROM.read(addr+5);
PotmM2s = EEPROM.read(addr+6);
PotmM2p = EEPROM.read(addr+7);
KbShft = EEPROM.read(addr+8);
PotmKprt = EEPROM.read(addr+9);
PotmKvib = EEPROM.read(addr+10);
PotmOw = EEPROM.read(addr+11);
PotmOm = EEPROM.read(addr+12);
PotmSw = EEPROM.read(addr+13);
PotmSm = EEPROM.read(addr+14);
//PotmAv1= EEPROM.read(addr+15);
PotmFf = EEPROM.read(addr+16);
PotmFm = EEPROM.read(addr+17);
PotmAv2= EEPROM.read(addr+18);
PotmAm = EEPROM.read(addr+19);
E1Fval=0;
E1Aval=255;
E2Fval=0;
E2Aval=255;
//Note=KbShft+ KbVal;
//GLIDE_NOTE=Note;
//LAST_NOTE=Note;
KEY_START =true;
}

ISR(TIMER0_COMPA_vect)// update tick for modulators+parameter update, SubOsc fixed mode, freq is 16E6/256=62k5
{
T0Div++;
if (T0Div >7){// freq =7812.5Hz
T0Div=0;
//make a midiclock interpolator, create 4 Ticks per midiclock tick ----------
if (MidiClk==true){
  if (MidiTick == true){
    MidiTick=false;
    Tick=true;
    Mclkper=Mclktim;
    Mclktim=0;
   }
  else if (Mclktim == Mclkper/4) {Tick=true;}
  else if (Mclktim == Mclkper/2) {Tick=true;}
  else if (Mclktim == Mclkper/2 + Mclkper/4){Tick=true;}
  Mclktim++;
}
//end of midiclock interpolator ----------------------------------------------
// LED indicators handling ---------------------------------------------------
TickDiv++;
if (LedPWMcnt >30){LedPWMcnt=0;}
else {LedPWMcnt++;}//PWMcounter runs from 0 to 31
if (Ledtab[LedPWMcnt] > E1val){PORTB = PORTB & B11101111;} //Led Modulator1 off
else {PORTB = PORTB | B00010000;}
if (Ledtab[LedPWMcnt] > E2val){PORTD = PORTD & B11101111;} // Led Modulator 2 off
else {PORTD = PORTD | B00010000;}
//if (LedPWMcnt >= LV1){PORTB = PORTB & B11111110;} //Led Value1 off
//else {PORTB = PORTB | B00000001;}
//if (LedPWMcnt >= LV2){PORTB = PORTB & B11111101;} // Led Value2 off
//else {PORTB = PORTB | B00000010;}

if (Ledtab[LedPWMcnt] > Sub<<4){PORTD = PORTD & B01111111;} // Led SubOsc off
else {PORTD = PORTD | B10000000;}
if (WavMode > 15){
   if (Ledtab[LedPWMcnt] > (DAC-8)<<4){PORTD = PORTD & B11111011;} // Led OSC wav off
   else {PORTD = PORTD | B00000100;}
}
else{
  if (Ledtab[LedPWMcnt] >= (PW)<<1){PORTD = PORTD & B11111011;} // Led OSC pulse off
  else if (SubAM !=0) {PORTD = PORTD | B00000100;} 
}
//end of LED indicators handling ---------------------------------------------------

if (SubAmp>0 || SubModeAM==true || SubFlm==true){

  if(PotmSw==0){// noise out
    Nstate=(Nstate<<1 | (((Nstate>>15)&1) ^((Nstate>>14)&1) ^ ((Nstate>>12)&1) ^ ((Nstate>>3)&1) ));  
    if (SubFlm==true){
    if (Nstate > 16000) {OCR0A=filterF-SubFlvl;}
    else {OCR0A=filterF;}
    }
    if(SubModeAM ==true){//AM on
    SubAM=Nstate;
      if (WavMode<16){//AM on pulse
        if (SubAM>15){DDRB=DDRB | B00000100;}//pinMode(10,OUTPUT);
        else {DDRB=DDRB&B11111011;}//pinMode(10,INPUT);
      }
   }
  else {Sub=Nstate & SubAmp;}
  }  
  else if (PotmSw<65){// pattern waves out
    if (SNoiseCnt > pgm_read_byte_near(SubNlen+PotmSw)){
      //Nstate=1+E2Fval/4;
      Nstate=1+E2Ccnt+LAST_VEL;
      SNoiseCnt=0;
     //SubSync=false;
    }
    else {SNoiseCnt++;}
   Nstate=(Nstate<<1 | (((Nstate>>15)&1) ^((Nstate>>14)&1) ^ ((Nstate>>12)&1) ^ ((Nstate>>3)&1) ));  
   SubDF=SubDF-(SubDF>>1)+(Nstate>>9);//digital lowpass filter
   Sub=SubDF>>1 & SubAmp;
//todo compare Nstate and decide SubBit is high or low  and do AM and FilterFM with that
    if (SubFlm==true)
    {
      if (Nstate >16000) {OCR0A=filterF-SubFlvl;}
      else {OCR0A=filterF;}
    }
    if(SubModeAM ==true){//AM on
    SubAM=SubDF;
      if (WavMode<16){//AM on pulse
        if (SubAM>15){DDRB=DDRB | B00000100;}//pinMode(10,OUTPUT);
        else {DDRB=DDRB&B11111011;}//pinMode(10,INPUT);
      }
   }
   else {
     SubAM=B01111111;
     Sub=SubDF & SubAmp;
   }
 }//end of: if PotmSw <65

 else if (PotmSw<88){// fix freq sub --------------------------------------
//SubPnt=SubPnt+ pgm_read_byte_near(SubSkip+(PotmSw-64));
if (SubSync==false) {
SubPnt=SubPnt+ pgm_read_byte_near(SubSkip+(PotmSw-65));
}
else {
SubPnt=!SubPnt;
SubSync=false;
}
Sub=pgm_read_byte_near(WavTab + SubPnt);

if (SubModeAM==true) {SubAM=Sub;}
else if (SubFlm==true){
if (Sub >48) {OCR0A=filterF-SubFlvl;}
   else {OCR0A=filterF;}
Sub=0;
}
else{Sub=Sub*PotmSm/32; }
//DACP=(DAC+Sub)&SubAM;//&SubAM;
//PORTC=DACP>>2;
}// End off fix freq sub --------------------------------------------
if (PotmSw<88){// if fix freq sub
   DACP=(DAC+Sub)&SubAM;//&SubAM;
   PORTC=DACP>>2;
}
}//end of if subamp>0
 else {
   Sub=0;
   }
 }
}//end of T0 int

ISR(TIMER1_COMPA_vect)//runs N times per cycle of the Note played. (N=1 for Pulse/PWM , N=8 for DAC,Noise)
{
PORTC=DACP>>2;

//Subosc: pulse oscillator on DAC, SubLen= cycle length,SubPls=pulse length, SubPnt=pointer,  SubBit= output bit 
if (WavMode <16){
 if (PotmSw>88){//sub tracks key
 if (SubPnt >= SubLen){SubPnt=0;} //reset pointer if >=SubLen 
  if (SubPnt<SubPls){
  //means SubBit=0;
  switch (SubMode){
  case 0:
  Sub=0; // sub > audio value set
  break;
  case 1:
  Sub=0; // sub > audio value ramp down
  break;
  case 2:
  OCR0A=filterF-SubFlvl;// if sub > filtermod on
  //OCR0A=(filterF>>1)+(filterF>>2);// if sub > filtermod on
  //OCR0A=filterF;// if sub > filtermod on
  break;
  case 3:
  DDRB=DDRB | B00000100;// if sub > AM
  break;
  }
  } // 
  else {
 // SubBit=1;
  switch (SubMode){
  case 0:
  Sub=SubAmp; // sub > audio value set
  break;
  case 1:
  Sub=SubAmp; // sub > audio value ramp down
  break;
  case 2:
  OCR0A=filterF;// if sub > filtermod on
  break;
  case 3:
  DDRB=DDRB&B11111011;// if sub > AM
  break;
  }
 }
  SubPnt++;


 }//end of if PotmSW>88 wavemode <2
else{//sub on fix freq
SubSync=true;
//  if (SubModeAM==true){//if sub > AM on
//     if (SubAM ==0){DDRB=DDRB&B11111011;}//pinMode(10,INPUT);
//   else {DDRB=DDRB | B00000100;}//pinMode(10,OUTPUT);
//    }

}
}//end of: if wavmode <2
else if (WavMode==16) {// WaveMode 2 

SubDiv++;
if (SubDiv >7){
  SubDiv=0;
  SubDivOct++;
  
if (SubDivOct > SubDivtab[oct]){
SubDivOct=0;// to do: combine with subskip oct
 if (PotmSw>88){
 if (SubPnt >= SubLen){SubPnt=0;} //
  if (SubPnt<SubPls){//Subbit=0
  //means SubBit=0;
   if (SubFlm==true){ OCR0A=filterF-SubFlvl;}// if sub > filtermod on
   //if (SubFlm==true){ OCR2A=VCA1-PotmSm+64;}// if sub > filtermod on
  SubAM=B01111111;  // set sub audio enable on
  Sub=0; // sub > audio value is zero
   } // end of Subbit=0
  else {//Subbit=1
  //SubBit=1;
  if (SubModeAM==true){ SubAM=B00011111;}//if sub > AM on
  else if (SubFlm==true){ OCR0A=filterF;}// if sub > filtermod on
  // else if (SubFlm==true){ OCR2A=VCA1;}//VCA1 on
  else { Sub=SubAmp;} // sub > audio value set
  } //end of Subbit=1
  
SubPnt++;//increase counter for subwave 
 }//end of if PotmSW>88 for wavemode=2
}//end of if SubDivOct 
}//end of if SubDiv >7

} //end of wavmode 2 handling
else{ //wavmode 3,4 handling
SubDiv++;
if (SubDiv >7){SubDiv=0;// to do: combine with subskip oct
 if (PotmSw>88){
if (SubPnt >= SubLen){SubPnt=0;} //SubLen is length of sub waveform, SubPnt is pointer
  if (SubPnt<SubPls){// Subbit=0
  //SubBit=0;
   if (SubModeAM==true) {SubAM=B00111111;}
  // if (SubModeAM==true && WavMode<16){//if sub > AM on
 //      DDRB=DDRB | B00000100;//pinMode(10,OUTPUT);
 //     }
   else if (SubFlm==true){ OCR0A=filterF-SubFlvl;}// if sub > filtermod on
   //SubAM=B11111111;  // set sub audio enable on
   Sub=0; // sub > audio value is zero
  } 
  else {// Subbit=1
  //SubBit=1;
  if (SubModeAM==true){//if sub > AM on
     SubAM=B00001111;
   //  DDRB=DDRB&B11111011;//pinMode(10,INPUT);
    }
   else if (SubFlm==true){ OCR0A=filterF;}// if sub > filtermod on
   else { Sub=SubAmp;} // sub > audio value set
 }
  
SubPnt++;
}//end of if PotmSW>88

}//end of if SubDiv >7
}//end of wavmode 3,4 handling
// end of SubOsc

  switch (WavMode){
  case 0:// osc off 
  DAC=0;
  break;
  case 1: // Pulse only
  DAC=0;
  break;
  case 16: //send 256...8 length x7bit wave from table WavTab to PORT C
  if (WavSkip >= WavSkiptab[oct]){
    LAST_WavP=WavP;
    WavP=WavP+WavInctab[oct];  
    WavP2=PW+WavP;//+WavInctab[oct]; 
    WavSkip=0;
    if (WavP<LAST_WavP){
    WavSelD=WavSel;// update waveform select on start waveform
    SubSync=true;
    }
  }
  else {WavSkip++;}

LP=pgm_read_byte_near(WavTab + WavP + WavSelD);
HP=pgm_read_byte_near(WavTab + WavP2 + WavSelD);
DAC= (HP+LP)>>2;
 break;
  case 28: // PRBS used to create variable length looped noise
   if (NoiseCnt > NoiseLen){
     state=1+E1Ccnt+LAST_VEL;
     //state=1+E2Fval/4;
     NoiseCnt=0;
    }
    else {NoiseCnt++;}
    state=(state<<1 | (((state>>15)&1) ^((state>>14)&1) ^ ((state>>12)&1) ^ ((state>>3)&1) ));
    // state variable filter:
   LP = LP + ((F1 * BP)>>7);
   HP =(state>>11) - LP - BP;
   BP = ((F1 * HP)>>7) + BP;
   DAC=32+((LP+HP)>>1);
    // DAC=state>>8;
    break;

   case 29:
 // 16 bit PRBS with shift register and XOR on Bit 16,15,13,4  makes white noise
state=(state<<1 | (((state>>15)&1) ^((state>>14)&1) ^ ((state>>12)&1) ^ ((state>>3)&1) ));
LP = LP + ((F1 * BP)>>7);
HP =(state>>11) - LP - BP;
BP = ((F1 * HP)>>7) + BP;
DAC=32+LP+HP;
   break;
   case 30:
 // 16 bit PRBS with shift register and XOR on Bit 16,15,13,4  makes white noise
state=(state<<1 | (((state>>15)&1) ^((state>>14)&1) ^ ((state>>12)&1) ^ ((state>>3)&1) ));
LP = LP + ((F1 * BP)>>7);
//HP =(state>>11)  - LP - ((Q1 * BP)>>5);
HP =(state>>11)  - LP - BP;
BP = ((F1 * HP)>>7) + BP;
DAC=32+LP;
   break;
   case 31:
 // 16 bit PRBS with shift register and XOR on Bit 16,15,13,4  makes white noise
state=(state<<1 | (((state>>15)&1) ^((state>>14)&1) ^ ((state>>12)&1) ^ ((state>>3)&1) ));
LP = LP + ((F1 * BP)>>7);
HP =(state>>11) - LP - BP;
BP = ((F1 * HP)>>7) + BP;
DAC=32+BP;
   break;
   case 32:
 // 16 bit PRBS with shift register and XOR on Bit 16,15,13,4  makes white noise
state=(state<<1 | (((state>>15)&1) ^((state>>14)&1) ^ ((state>>12)&1) ^ ((state>>3)&1) ));
LP = LP + ((F1 * BP)>>7);
HP =(state>>11)  - LP - BP;
BP = ((F1 * HP)>>7) + BP;
DAC=32+HP;
   break;
   default:
   DAC=state>>9;
   break;

// break;
}
    DACP=(DAC+Sub)&SubAM;
    
}
